==> json-io-4.3.0-sources.jar: com/cedarsoftware/util/io/FastPushbackReader.java <==
package com.cedarsoftware.util.io;

import java.io.FilterReader;
import java.io.IOException;
import java.io.Reader;

/**
 * This class adds significant performance increase over using the JDK
 * PushbackReader.  This is due to this class not using synchronization
 * as it is not needed.
 */
public class FastPushbackReader extends FilterReader
{
    private static final int SNIPPET_LENGTH = 200;
    private final int[] buf;
    private final int[] snippet;
    private int idx;
    protected int line;
    protected int col;
    private int snippetLoc = 0;

    FastPushbackReader(Reader reader, int size)
    {
        super(reader);
        if (size <= 0)
        {
            throw new JsonIoException("size <= 0");
        }
        buf = new int[size];
        idx = size;
        snippet = new int[SNIPPET_LENGTH];
        line = 1;
        col = 0;
    }

    FastPushbackReader(Reader r)
    {
        this(r, 1);
    }

    String getLastSnippet()
    {
        StringBuilder s = new StringBuilder();
        for (int i=snippetLoc; i < SNIPPET_LENGTH; i++)
        {
            if (appendChar(s, i))
            {
                break;
            }
        }
        for (int i=0; i < snippetLoc; i++)
        {
            if (appendChar(s, i))
            {
                break;
            }
        }
        return s.toString();
    }

    private boolean appendChar(StringBuilder s, int i)
    {
        try
        {
            if (snippet[i] == 0)
            {
                return true;
            }
            s.appendCodePoint(snippet[i]);
        }
        catch (Exception e)
        {
            return true;
        }
        return false;
    }

    public int read() throws IOException
    {
        final int ch = idx < buf.length ? buf[idx++] : in.read();
        if (ch >= 0)
        {
            if (ch == 0x0a)
            {
                line++;
                col = 0;
            }
            else
            {
                col++;
            }
            snippet[snippetLoc++] = ch;
            if (snippetLoc >= SNIPPET_LENGTH)
            {
                snippetLoc = 0;
            }
        }
        return ch;
    }

    public void unread(int c) throws IOException
    {
        if (idx == 0)
        {
            throw new JsonIoException("unread(int c) called more than buffer size (" + buf.length + ").  Increase FastPushbackReader's buffer size.  Currently " + buf.length);
        }
        if (c == 0x0a)
        {
            line--;
        }
        else
        {
            col--;
        }
        buf[--idx] = c;
        snippetLoc--;
        if (snippetLoc < 0)
        {
            snippetLoc = SNIPPET_LENGTH - 1;
        }
        snippet[snippetLoc] = c;
    }
}

==> json-io-4.5.0-sources.jar: com/cedarsoftware/util/io/FastPushbackReader.java <==
package com.cedarsoftware.util.io;

import java.io.FilterReader;
import java.io.IOException;
import java.io.Reader;

/**
 * This class adds significant performance increase over using the JDK
 * PushbackReader.  This is due to this class not using synchronization
 * as it is not needed.
 */
public class FastPushbackReader extends FilterReader
{
    private static final int bufsize = 256;
    private final int[] buf;
    private int idx;
    private int unread = Integer.MAX_VALUE;
    protected int line;
    protected int col;

    FastPushbackReader(Reader reader)
    {
        super(reader);
        buf = new int[bufsize];
        idx = 0;
        line = 1;
        col = 0;
    }

    String getLastSnippet()
    {
        StringBuilder s = new StringBuilder();
        for (int i=idx; i < bufsize; i++)
        {
            if (appendChar(s, i))
            {
                break;
            }
        }
        for (int i=0; i < idx; i++)
        {
            if (appendChar(s, i))
            {
                break;
            }
        }
        return s.toString();
    }

    private boolean appendChar(StringBuilder s, int i)
    {
        try
        {
            final int snip = buf[i];
            if (snip == 0)
            {
                return true;
            }
            s.appendCodePoint(snip);
        }
        catch (Exception e)
        {
            return true;
        }
        return false;
    }

    public int read() throws IOException
    {
        int ch = unread == Integer.MAX_VALUE ? in.read() : unread;
        unread = Integer.MAX_VALUE;

        buf[idx++] = ch;
        if (idx >= bufsize)
        {
            idx = 0;
        }

        if (ch == 0x0a)
        {
            line++;
            col = 0;
        }
        else
        {
            col++;
        }
        return ch;
    }

    public void unread(int c) throws IOException
    {
        unread = c;
        if (idx < 1)
        {
            idx = bufsize - 1;
        }
        else
        {
            idx--;
        }

        if (c == 0x0a)
        {
            line--;
        }
        else
        {
            col--;
        }
    }
}
